const UNBUNDLED = "unbundled"; 

var bundlesDOM = null;
var tagsDOM = null;
var bookmarksDOM = null;

var myObserver = {
  observe: function(subject, topic, data) {
    switch (topic) {
      case "ext:delicious:defaultUserChanged":      
        reloadTags();
        break;
      case "ext:delicious:tagxmlForDefaultUser":
      	if (tagsDOM != null) {
					clearTags();      
				}
				
        tagsDOM = new DOMParser().parseFromString(data, "text/xml");               
        loadTags();
        break;
      case "ext:delicious:bundlesxmlForDefaultUser":        
        bundlesDOM = new DOMParser().parseFromString(data, "text/xml");
        break;        
      case "ext:delicious:reloadBookmarks":
        loadSelectedTag();
        break;                
      case "ext:delicious:sidebarSetLoadedBookmarks":
      	bookmarksDOM = new DOMParser().parseFromString(data, "text/xml");
        setLoadedBookmarks();
        break;                
    }
  }
};

/**
 * When the sidebar is first loaded, load the user's tags.
 */
function onLoad() {
    var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
    
    observerService.addObserver(myObserver,"ext:delicious:defaultUserChanged",false);
    observerService.addObserver(myObserver,"ext:delicious:tagxmlForDefaultUser",false);
    observerService.addObserver(myObserver,"ext:delicious:bundlesxmlForDefaultUser",false);
    observerService.addObserver(myObserver,"ext:delicious:sidebarSetLoadedBookmarks",false);    
    observerService.addObserver(myObserver,"ext:delicious:reloadBookmarks",false);        
    
    // Send message that sidebar is loaded
    observerService.notifyObservers(null,"ext:delicious:sidebarLoaded","");
    
    window.onunload = function() {
      observerService.removeObserver(myObserver,"ext:delicious:defaultUserChanged");
      observerService.removeObserver(myObserver,"ext:delicious:tagxmlForDefaultUser");
      observerService.removeObserver(myObserver,"ext:delicious:bundlesxmlForDefaultUser");
      observerService.removeObserver(myObserver,"ext:delicious:sidebarSetLoadedBookmarks");
      observerService.removeObserver(myObserver,"ext:delicious:reloadBookmarks");      
    };
    
    init();
}

function init() {
	toggleShowSearchBar();
	toggleShowTagList();
	toggleShowTagCloud();
	toggleShowBookmarks();
}

/**
 * Retrieves user's tags from the del.icio.us server.
 */
function loadTags() {
    try {   
        var tagsTree = document.getElementById('delicious-sidebar-tags-tree');
        var xmlDOM = tagsDOM;
        if (xmlDOM == false || xmlDOM == null) {
            return;
        }                

        var root = xmlDOM.documentElement;
        var myTags = root.getElementsByTagName("tag");
        
        var RDF = Components.classes['@mozilla.org/rdf/rdf-service;1'].getService();
        RDF = RDF.QueryInterface(Components.interfaces.nsIRDFService);
        
        var RDFC = Components.classes['@mozilla.org/rdf/container;1'].getService();
        RDFC = RDFC.QueryInterface(Components.interfaces.nsIRDFContainer);
        
        var RDFCUtils = Components.classes['@mozilla.org/rdf/container-utils;1'].getService();
        RDFCUtils = RDFCUtils.QueryInterface(Components.interfaces.nsIRDFContainerUtils);
        
        var ds = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"].createInstance();
        ds = ds.QueryInterface(Components.interfaces.nsIRDFDataSource);
        
        // create our root nodes
        var rootNode = RDF.GetResource("urn:root");
        var sequenceNode = RDF.GetResource("urn:root:seq");
        // Root node in tree
        ds.Assert(rootNode, RDF.GetResource("http://delicious.mozdev.org/RDF#tags"), sequenceNode, true);
        
        RDFCUtils.MakeSeq(ds, sequenceNode);
        RDFC.Init(ds, sequenceNode);       
        
        tagsTree.database.AddDataSource(ds);
        
        for (var i = 0; i < myTags.length; i++) {
            var tagName = myTags[i].getAttribute("tag");
            var count = myTags[i].getAttribute("count");
            var countSort = leftPad(myTags[i].getAttribute("count"));
            
            var newURI = "urn:root:file:" + i;

            // add the new element
            RDFC.AppendElement(RDF.GetResource(newURI));
            
            ds.Assert(RDF.GetResource(newURI),
                RDF.GetResource("http://delicious.mozdev.org/RDF#name"),
                RDF.GetLiteral(tagName), true);
            
            ds.Assert(RDF.GetResource(newURI),
                RDF.GetResource("http://delicious.mozdev.org/RDF#count"),
                RDF.GetLiteral(count), true);
            
            // "countSort" is used as a hidden column that allows the "count"
            // column to be sorted properly (numerically instead of
            // lexically). Padding the number makes the two equivalent, but
            // you don't want to *see* the padded version.
            ds.Assert(RDF.GetResource(newURI),
                RDF.GetResource("http://delicious.mozdev.org/RDF#countSort"),
                RDF.GetLiteral(countSort), true);
              
            // needed for cloud view
           	addTagToBundle(tagName, count);
        }
        tagsTree.builder.rebuild();
        
        highlightTags();
    }
    catch(e) { alert("Error loading tags: " + e); }
}

function leftPad(number) {
    var padded=number;
    while(padded.length<5) padded="0"+padded;
    return(padded);
}

function clearTags() {
    var tree = document.getElementById('delicious-sidebar-tags-tree');
    clearTree(tree);
    
    clearTagCloudBox();
}

function reloadTags() {   
		var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
    
    clearTags();
    observerService.notifyObservers(null,"ext:delicious:sidebarReloadTagsRequest","");
    
    clearFeeds();
    observerService.notifyObservers(null,"ext:delicious:sidebarReloadBookmarksRequest","");
}

function clearFeeds() {
    var tree = document.getElementById('delicious-sidebar-bookmarks-tree');
    clearTree(tree);
}

/**
 * Fix for getCellText. In Firefox <1.5 it took the column as a string, in Firefox 1.5beta onwards as a Column.
 */
function columnFromTree(aTree, aColumnName) {
  if (aTree.columns) {
    return aTree.columns[aColumnName];  // Firefox 1.5+
  } else {
    return aColumnName;  // Firefox <1.5
  }
}

/**
 * Queries the selected tags and loads the results into the bookmark list.
 */
function loadSelectedTag() {   
    const tagsTree = document.getElementById('delicious-sidebar-tags-tree');
    if(tagsTree.view.selection.count == 0) {
        return false;
    } else {
        window.status="Loading tags...";
        var start = new Object();
        var end = new Object();
        var numRanges = tagsTree.view.selection.getRangeCount();
        var tags = "";
        var column = columnFromTree(tagsTree, "name_column");

        // Discontiguous selections are broken up into ranges, so 
        // we loop over each range in turn.
        for(var t=0;t<numRanges;t++) {
            tagsTree.view.selection.getRangeAt(t,start,end);
            for(var v=start.value;v<=end.value;v++) {
                tags+=tagsTree.view.getCellText(v, column) + "+";
                // Once the RSS feed is fixed, this needs to be enabled
                // in order to allow tags like "C++".
                //tags+="\""+tagsTree.view.getCellText(v, "name") + "\"+";
            }
        }

        tags=tags.substring(0,tags.length-1);

        loadFeed(tags);
        window.status="Done";
        return true;
    }
}

/**
 * Query del.icio.us for bookmarks under the specified tags and load
 * the tags into the bookmarks tree.
 */
function loadFeed(tags) {
    var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
    observerService.notifyObservers(null,"ext:delicious:sidebarLoadBookmarksRequest", tags);
}


/**
 * Fills the tree with bookmarks after they were retrieved by the deliciousService.
 */
function setLoadedBookmarks() {
  if (bookmarksDOM == false || bookmarksDOM == null) {
  	return;
  }
    
	try	{		
			var bookmarksTree = document.getElementById('delicious-sidebar-bookmarks-tree');

			highlightTags();
		
			var RDF = Components.classes['@mozilla.org/rdf/rdf-service;1'].getService();
			RDF = RDF.QueryInterface(Components.interfaces.nsIRDFService);

			var RDFC = Components.classes['@mozilla.org/rdf/container;1'].getService();
			RDFC = RDFC.QueryInterface(Components.interfaces.nsIRDFContainer);

			var RDFCUtils = Components.classes['@mozilla.org/rdf/container-utils;1'].getService();
			RDFCUtils = RDFCUtils.QueryInterface(Components.interfaces.nsIRDFContainerUtils);

			var ds = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"].createInstance();
			ds = ds.QueryInterface(Components.interfaces.nsIRDFDataSource);

			// create our root nodes
			var rootNode = RDF.GetResource("urn:root");
			var sequenceNode = RDF.GetResource("urn:root:seq");
			// Root node in tree
			ds.Assert(rootNode, RDF.GetResource("http://delicious.mozdev.org/RDF#bookmarks"), sequenceNode, true);

			RDFCUtils.MakeSeq(ds, sequenceNode);
			RDFC.Init(ds, sequenceNode);

			// Before setting the new data source
			// get rid of all old ones
			clearFeeds();
			bookmarksTree.database.AddDataSource(ds);

			var nsResolver = bookmarksDOM.createNSResolver( bookmarksDOM.ownerDocument == null ? bookmarksDOM.documentElement : bookmarksDOM.ownerDocument.documentElement);
			var bookmarks = bookmarksDOM.evaluate("//post", bookmarksDOM, null, window.XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);

			for ( var i=0 ; i < bookmarks.snapshotLength; i++ ) {			
					var title = bookmarks.snapshotItem(i).getAttribute("description");
					var extended = bookmarks.snapshotItem(i).getAttribute("extended");
					var url = bookmarks.snapshotItem(i).getAttribute("href");
							
				  // the two others are required fields!
					if (extended == null) {
						extended = "";
					}
				
					var newURI = "urn:root:file:" + i;

					// add the new element
					RDFC.AppendElement(RDF.GetResource(newURI));

					ds.Assert(RDF.GetResource(newURI),
							RDF.GetResource("http://delicious.mozdev.org/RDF#title"),
							RDF.GetLiteral(title), true);

					ds.Assert(RDF.GetResource(newURI),
							RDF.GetResource("http://delicious.mozdev.org/RDF#description"),
							RDF.GetLiteral(extended), true);
							
					ds.Assert(RDF.GetResource(newURI),
							RDF.GetResource("http://delicious.mozdev.org/RDF#url"),
							RDF.GetLiteral(url), true);							
			}
			bookmarksTree.builder.rebuild();						
	}
	catch(e) { alert("Error loading bookmarks: "+e); }
  
}

//
// Load link
//
function openSingleBookmark(event, isDblClick) {
		var tree = document.getElementById('delicious-sidebar-bookmarks-tree');
		var column = columnFromTree(tree, "link_column");
		var bookmarkUrl = tree.view.getCellText(tree.currentIndex, column);

		if (isDblClick) {			
			getContentBrowser().loadURI(bookmarkUrl);			
    } else if (event.ctrlKey || (event.button == 1)) {
			var newTab = getContentBrowser().addTab(bookmarkUrl);
			getContentBrowser().selectedTab = newTab;
    } 
    
    return;
}

function openMultipleBookmarks(loadFirstBookmarkInNewTab) {
    const bmsTree = document.getElementById('delicious-sidebar-bookmarks-tree');
    var start = new Object();
    var end = new Object();
    var numRanges = bmsTree.view.selection.getRangeCount();
    var tags = "";
    var first = true;
    var column = columnFromTree(bmsTree, "link_column");

    for(var t=0;t<numRanges;t++) {
			bmsTree.view.selection.getRangeAt(t,start,end);
			for(var v=start.value;v<=end.value;v++) {
				var bookmarkUrl = bmsTree.view.getCellText(v, column);
				if(first && !loadFirstBookmarkInNewTab) {
					getContentBrowser().loadURI(bookmarkUrl);	
					first=false;
				} else {
					getContentBrowser().addTab(bookmarkUrl);
				}
			}
    }
}

function deleteBookmarks() {
    
    const bmsTree = document.getElementById('delicious-sidebar-bookmarks-tree');
    var start = new Object();
    var end = new Object();
    var numRanges = bmsTree.view.selection.getRangeCount();
    var tags = "";
    var first = true;
    var column = columnFromTree(bmsTree, "link_column");

		var lastItem = bmsTree.view.rowCount == 1;

    for(var t=0;t<numRanges;t++) {
			bmsTree.view.selection.getRangeAt(t,start,end);
			for(var v=start.value;v<=end.value;v++) {
				var bookmarkUrl = bmsTree.view.getCellText(v, column);

				var answer = confirm("Delete bookmark with url '" + bookmarkUrl +"'?");
				if(answer) {
						var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
						observerService.notifyObservers(null,"ext:delicious:sidebarDeleteBookmarkRequest", bookmarkUrl);

						if (lastItem) {
							reloadTags();		            
						}		            
						return true;
				}
				else {
						return false;
				}
			}
    }
    return false;
}

//
// Rename tag
//
function renameTag() {
	const tagsTree = document.getElementById('delicious-sidebar-tags-tree');
	var column = columnFromTree(tagsTree, "name_column");
	const tag = tagsTree.view.getCellText(tagsTree.currentIndex, column);
	const count = tagsTree.view.selection.getRangeCount();

	if(count == 0) {
		alert("No tag selected to rename");
		return false;
	} else {
			var name = prompt("Enter new name for tag '" + tag + "'");
			if(name.length > 0) {
					var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
					observerService.notifyObservers(null,"ext:delicious:sidebarRenameTagRequest","from:"+tag+"&&__to:"+name);
					return true;
			} else {
					return false;
			}
	}
}

//
// Removes all datasources from a tree
//
function clearTree(tree) {
	var datasources = tree.database.GetDataSources();
	while(datasources.hasMoreElements()){
		var datasource = datasources.getNext();
		datasource = datasource.QueryInterface(Components.interfaces.nsIRDFDataSource);
		tree.database.RemoveDataSource(datasource);
	}

	tree.builder.rebuild();
}

function populateBookmarkToolTip(event) {		
	const bmsTree = document.getElementById('delicious-sidebar-bookmarks-tree');
	var row = {}, column = {}, part = {};
	var boxobject = bmsTree.boxObject;
	
	boxobject.QueryInterface(Components.interfaces.nsITreeBoxObject);
	
  var rowIndex = boxobject.getRowAt(event.clientX, event.clientY);
  
  if (rowIndex < 0) {
  	return;
  }
  
  var title = bmsTree.view.getCellText(rowIndex, bmsTree.columns.getColumnAt(0));
	var descr = bmsTree.view.getCellText(rowIndex, bmsTree.columns.getColumnAt(1));
	var url   = bmsTree.view.getCellText(rowIndex, bmsTree.columns.getColumnAt(2));
	
	var bookmarkToolTipTitle = document.getElementById("bookmarkToolTipTitle");  
	var bookmarkToolTipDescr = document.getElementById("bookmarkToolTipDescr");
	var bookmarkToolTipURL = document.getElementById("bookmarkToolTipURL");

		
	bookmarkToolTipTitle.setAttribute("value", title);
	bookmarkToolTipDescr.setAttribute("value", descr);
	bookmarkToolTipURL.setAttribute("value", url);
}

function editBookmarks() {
    const bmsTree = document.getElementById('delicious-sidebar-bookmarks-tree');
    var start = new Object();
    var end = new Object();
    var numRanges = bmsTree.view.selection.getRangeCount();
    var columnLink = columnFromTree(bmsTree, "link_column");
    var columnTitle = columnFromTree(bmsTree, "title_column");

    for(var t=0;t<numRanges;t++) {
        bmsTree.view.selection.getRangeAt(t,start,end);
        for(var v=start.value;v<=end.value;v++) {
            var bookmarkUrl = bmsTree.view.getCellText(v, columnLink);
            var title = bmsTree.view.getCellText(v, columnTitle);
								
						// is necessary because if add-dialog is called to post the url is encoded.
						// so it is best to create the same prerequisite!
						var url = encodeURIComponent(bookmarkUrl);		
								
						// load the add dialog with the bookmarkUrl
            var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
            observerService.notifyObservers(null,"ext:delicious:editBookmarkRequest", url + ";;;" + title);						
        }
    }

    return;
}

/**
 * Method is called if a tag was selected.
 * (Re)loads the associated bookmarks for all selected tags.
 */
function tagLinkClick(tag) {
  var currentlySelectedTags = getCurrentlySelectedTags();

	var clickOnAnAlreadySelectedTag = false;
	var splitCurrentlySelectedTags = currentlySelectedTags.split(" ");
	for (var i = 0; i < splitCurrentlySelectedTags.length; i++) {
		if (splitCurrentlySelectedTags[i] == tag) {
			clickOnAnAlreadySelectedTag = true;
			break;
		}
	}

	if (clickOnAnAlreadySelectedTag) {
		selectedTagClick(tag);
	} else {
		var clickOfANotAssociatedTag = false;
		if (currentlySelectedTags.length > 0) {
			var clickedLabel = document.getElementById(tag);
			var styleAttrVal = clickedLabel.getAttribute("style");
			if (styleAttrVal.indexOf("color") >= 0) {
				var startIndex = styleAttrVal.indexOf("color");
				var beginning = styleAttrVal.substring(0, startIndex - 1);
				var restWithColor = styleAttrVal.substring(startIndex);
				// color not really correct, can have leading space
				var color = restWithColor.substring(6, restWithColor.indexOf(";"));

				if (color.indexOf("blue") >= 0) {
					clickOfANotAssociatedTag = true;
				}
			} else {
				clickOfANotAssociatedTag = true;
			}
		}

		if (clickOfANotAssociatedTag) {
			deselectAllSelectedTags();
			currentlySelectedTags = getCurrentlySelectedTags();
		}

		// the marking has to take place before the bookmarks are loaded!
		markTagAsSelected(tag);		

		var tags = tag + (currentlySelectedTags.length > 0 ? " " + currentlySelectedTags : "");
		loadFeed(tags);	
	}
}

/**
 * Method is called if a tag was deselected.
 * (Re)loads the associated bookmarks for all selected tags.
 */
function selectedTagClick(tag) {
	var selectedTagsBox = document.getElementById("selectedTagsBox");
	selectedTagsBox.removeChild(document.getElementById("selected" + tag));

  var currentlySelectedTags = getCurrentlySelectedTags();
  
	if (currentlySelectedTags != null && currentlySelectedTags.length > 0) {
		loadFeed(currentlySelectedTags);		
	} else {		
		var bookmarksTree = document.getElementById('delicious-sidebar-bookmarks-tree');
		clearTree(bookmarksTree);		
		this.bookmarksDOM = null;
	}
	
	this.highlightTags();
}

/**
 * Marks the clicked tag as selected.
 * Currently it shows an extra box with the selected tags. But it might be better to highlight them by a different color.
 */
function markTagAsSelected(tag) {
	var selectedTagsBox = document.getElementById("selectedTagsBox");
	var tagLabel  = document.createElement("label");
	
	tagLabel.setAttribute("id", "selected" + tag);
	tagLabel.setAttribute("value", tag);
	tagLabel.setAttribute("class", "text-link");
	tagLabel.setAttribute("onclick", "selectedTagClick(\"" + tag +"\");");
	
	selectedTagsBox.appendChild(tagLabel);	
}

/**
 * Answers all currently selected tags.
 */
function getCurrentlySelectedTags() {
	var selectedTagsBox = document.getElementById("selectedTagsBox");

	var tagLabels = selectedTagsBox.getElementsByTagName("label");

	var tags = "";
	if (tagLabels != null && tagLabels.length > 0) {				  
		for( var i = 0; i < tagLabels.length; i++ ){
		    tags += tagLabels[i].getAttribute("value") + " ";
  	}		
  	tags = tags.slice(0, -1);
	}
	
	return tags;
}

/**
 * Highlights tags the selected can combined with.
 */
function highlightTags() {
	var selectedTags = getCurrentlySelectedTags();
	var splitSelectedTags = selectedTags.split(" ");
	var groupBox = document.getElementById("tagsCloudChildrenBox");
	var boxes = groupBox.getElementsByTagName("hbox");
	
	if (bookmarksDOM != null) {
		var nsResolver = bookmarksDOM.createNSResolver(bookmarksDOM.ownerDocument == null ? bookmarksDOM.documentElement : bookmarksDOM.ownerDocument.documentElement);
		var bookmarks = bookmarksDOM.evaluate("//post", bookmarksDOM, null, window.XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);	
	} 
	
	for (var i = 0; i < boxes.length; i++) {
		var labels = boxes[i].getElementsByTagName("label");
		
		for (var j = 0; j < labels.length; j++) {
			var label = labels[j];
			
			if (label == null) {
				continue;
			}
			
			var isBundle = (label.getAttribute("class") == "bundleLabel");

			if (isBundle) {
				// bundles are not highlighted!				
				continue;
			}			

			var val = label.value;
			var mustBeHighlighted = false;
			
			if (bookmarks != null) {
				for (var k = 0; k < bookmarks.snapshotLength; k++) {
				
					var tagsOfBookmark = bookmarks.snapshotItem(k).getAttribute("tag");
					var splitTagsOfBookmark = tagsOfBookmark.split(" ");								

					for (var l = 0; l < splitTagsOfBookmark.length; l++) {
						if (splitTagsOfBookmark[l] == val) {												
							setColorToLabel(label, "green");
							mustBeHighlighted = true;
							break;
						}											
					}				

					for (var l = 0; l < splitSelectedTags.length; l++) {
						if (splitSelectedTags[l] == val) {					
							setColorToLabel(label, "red");
							mustBeHighlighted = true;
							break;
						}						
					}		
				}
			} 
			
			if (!mustBeHighlighted) {						  
				setColorToLabel(label, "blue");
			}
		}
	}
}

function setColorToLabel(label, color) {
	var styleAttrVal = label.getAttribute("style");

	var styleValueToSet = "";
	if (styleAttrVal == null) {
		styleValueToSet = "color: " + color + ";";
		
	} else {
		if (styleAttrVal.indexOf("color") >= 0) {				
			// remove old color from styleAttrVal
			var startIndex = styleAttrVal.indexOf("color");
			var beginning = styleAttrVal.substring(0, startIndex - 1);
			var restWithColor = styleAttrVal.substring(startIndex);
			var end = restWithColor.substring(restWithColor.indexOf(";") + 1);
			
			styleAttrVal = beginning + end;			
		} 
		
		styleValueToSet = styleAttrVal + "color: " + color + ";";		
	}
	
	label.setAttribute("style", styleValueToSet);
}

function deselectAllSelectedTags() {	
	var currentTags = getCurrentlySelectedTags();
	if (currentTags.length > 0) {
		var splittedTags = currentTags.split(" ");		
		for (var i = 0; i < splittedTags.length; i++) {		 
			var selectedTagsBox = document.getElementById("selectedTagsBox");
			selectedTagsBox.removeChild(document.getElementById("selected" + splittedTags[i]));		
		}		

		var bookmarksTree = document.getElementById('delicious-sidebar-bookmarks-tree');
		bookmarksDOM = null;
		clearTree(bookmarksTree);
		
		highlightTags();
	}
}

function addLinkRow(parentBox, curHBox, bundle) {
	var newHBox  = document.createElement("hbox");

	newHBox.id = "currentTagLinkBox" + bundle;
	curHBox.id = "";
	
	newHBox.setAttribute("style", "-moz-box-align:baseline;")
	parentBox.insertBefore(newHBox, curHBox.nextSibling);

	return newHBox;
}

function addTagToBundle(tagname, count) {
	var bundles = getBundleForTag(tagname);
	
	for (var i = 0; i < bundles.length; i++) {
		addTagToOneBundle(tagname, count, bundles[i]);
	}
}

function addTagToOneBundle(tagname, count, bundle) {  
  var parentBox = document.getElementById("tagsCloudChildrenBox");
	var	bundleComponent = document.getElementById("bundle" + bundle); 	

	var curHBox = null;		
	if (bundleComponent != null) {
		// Bundle already exists		
		curHBox = document.getElementById("currentTagLinkBox" + bundle);
	} else {
		// Create new Bundle
		bundleComponent = document.createElement("vbox");	
		bundleComponent.setAttribute("id", "bundle" + bundle);
		//parentBox.appendChild(bundleComponent); 

		var nextBundle = getBundleToInsertBefore(bundle);	  
		parentBox.insertBefore(bundleComponent, nextBundle);

		// Add first line box
		curHBox  = document.createElement("hbox");
		curHBox.setAttribute("id", "currentTagLinkBox" + bundle);
		curHBox.setAttribute("style", "-moz-box-align:baseline;");
		bundleComponent.appendChild(curHBox);

		// Add bundle name
		var bundleLink = document.createElement("label");
		bundleLink.setAttribute("value", bundle);  
		bundleLink.setAttribute("class", "bundleLabel");
		curHBox.appendChild(bundleLink);
	}

	var newLink  = document.createElement("label");

	// remember the oldwidth
	oldWidth = parentBox.boxObject.width;

	var fontsize = getFontSizeForTag(count);

	newLink.setAttribute("id", tagname);
	newLink.setAttribute("value", tagname);
	newLink.setAttribute("class", "text-link");
	newLink.setAttribute("onclick", "tagLinkClick(\"" + tagname +"\");");
	newLink.setAttribute("style", "font-size: " + fontsize + "%;");

	curHBox.appendChild(newLink);

	if(isWrapTime(newLink, parentBox, oldWidth)) {
		newHBox = addLinkRow(bundleComponent, curHBox, bundle);
		newHBox.appendChild(newLink);
	}
}

/**
 * Gets the bundle for the given tag name or null.
 */
function getBundleForTag(tag) {
	if (!useBundles()) {	   
		return new Array(UNBUNDLED);
	}

	if (bundlesDOM != null) {
	  var retval = new Array();
		var nsResolver = bundlesDOM.createNSResolver(bundlesDOM.ownerDocument == null ? bundlesDOM.documentElement : bundlesDOM.ownerDocument.documentElement);
		var  bundles = bundlesDOM.evaluate("//bundle", bundlesDOM, null, window.XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);	
		
		if (bundles != null) {
		  var counter = 0;
			for (var k = 0; k < bundles.snapshotLength; k++) {
				var bundlename = bundles.snapshotItem(k).getAttribute("name");
				var tagsOfBundle = bundles.snapshotItem(k).getAttribute("tags");
				var splitTagsOfBundle = tagsOfBundle.split(" ");								

				for (var l = 0; l < splitTagsOfBundle.length; l++) {
					if (splitTagsOfBundle[l] == tag) {	
						retval[counter] = bundlename;												
						counter++;
					}					
				}					
			}
			if (counter > 0) {
				return retval;
			}
		}
	} 
	
	return new Array(UNBUNDLED);
}

/**
 * Answers the component the given bundle should be inserted before.
 */
function getBundleToInsertBefore(bundle) {
	if (bundle != null) {
		var groupBox = document.getElementById("tagsCloudChildrenBox");
		var boxes = groupBox.getElementsByTagName("hbox");

		for (var i = 0; i < boxes.length; i++) {
			var labels = boxes[i].getElementsByTagName("label");

			for (var j = 0; j < labels.length; j++) {
				var label = labels[j];

				if (label == null) {
					continue;
				}

				var isBundle = (label.getAttribute("class") == "bundleLabel");
				var val = label.value;

				if (isBundle && val > bundle) {				
				 	return boxes[i].parentNode;
				}		
			}
		}
	}
	return null;
}

/*
 * Need the old width in there because the box will automatically expand
 * not the best system to do this sort of thing with, but hopefully it
 * will do the job
 */
function isWrapTime(testLabel, testBox, oldWidth) {
	var isWrap = false;
	var padding = 22;

	if((testLabel.boxObject.x + testLabel.boxObject.width) > (testBox.boxObject.x+oldWidth-padding)){
		isWrap = true;
	}

	return isWrap;
}

/**
 * Resets the tag cloud view.
 */
function clearTagCloudBox() {    
	var remele = document.getElementById("tagsCloudChildrenBox");
	if (remele.parentNode) {
		remele.parentNode.removeChild(remele);
	}      
    
	var tagsCloudBox = document.getElementById("tagsCloudBox");
	var newParentBox = document.createElement("vbox");
	
	newParentBox.setAttribute("id", "tagsCloudChildrenBox");
	newParentBox.setAttribute("style", "overflow: -moz-scrollbars-vertical;");
	newParentBox.setAttribute("flex", "1");

	tagsCloudBox.insertBefore(newParentBox, tagsCloudBox.firstChild);
}

function getFontSizeForTag(count) {
  // Todo externalize these two constants so that they are changable in a dialog.
  var normalSize = 95;
  var maxSize = 140;
  var changeSizeBy = 2;
  
  var calculatedSize = normalSize - changeSizeBy + changeSizeBy * parseInt(count);
	return calculatedSize > maxSize ? maxSize : calculatedSize;
}

/**
 * Answers the current browser to open a tab or window or stuff.
 */
function getContentBrowser() {
	var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
	var topWindowOfType = windowManager.getMostRecentWindow("navigator:browser");
	if (topWindowOfType) {
		return topWindowOfType.document.getElementById('content');
	}
	return null;
}

function getCheckboxCheck(element_id) {
	var checkboxNode = document.getElementById(element_id);	
	return checkboxNode.getAttribute("checked") == "true";
}

function toggleShowSearchBar() {
/* TODO activate later
	var showSearchBar = getCheckboxCheck("chkShowSearchBar");
	document.getElementById("barSearch").hidden = !showSearchBar;
	*/
}

function toggleShowTagCloud() {
	var show = getCheckboxCheck("chkShowTagCloud");
	document.getElementById("delicious-tagcloud-splitter").hidden = !show;
	document.getElementById("tagsCloudBox").hidden = !show;	
}

function toggleShowTagList() {
	var show = getCheckboxCheck("chkShowTagList");
	document.getElementById("delicious-sidebar-splitter").hidden = !show;
	document.getElementById("delicious-sidebar-tags-tree").hidden = !show;	
}

function toggleShowBookmarks() {
	var show = getCheckboxCheck("chkShowBookmarks");
	document.getElementById("delicious-sidebar-bookmarks-tree").hidden = !show;	
	document.getElementById("delicious-sidebar-bookmarks").hidden = !show;	
}

function useBundles() {
	return getCheckboxCheck("chkUseBundles");
}

function openSettingDialog() {
	//TODO send an event to open the settings dialog
	alert("Coming soon...");
}


var aConsoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
function logMessage(message) {
	aConsoleService.logStringMessage("delicious: '" + message + "'");
}
